@@ -255,6 +255,22 @@ pub fn input_to_egui(
                 },
             });
         }
+        WindowEvent::HoveredFile(path) => {
+            input_state.raw.hovered_files.push(egui::HoveredFile {
+                path: Some(path.clone()),
+                ..Default::default()
+            });
+        }
+        WindowEvent::HoveredFileCancelled => {
+            input_state.raw.hovered_files.clear();
+        }
+        WindowEvent::DroppedFile(path) => {
+            input_state.raw.hovered_files.clear();
+            input_state.raw.dropped_files.push(egui::DroppedFile {
+                path: Some(path.clone()),
+                ..Default::default()
+            });
+        }
         _ => {
             // dbg!(event);
         }
@@ -504,7 +520,7 @@ impl EguiGlium {
     pub fn on_event(&mut self, event: &glium::glutin::event::WindowEvent<'_>) {
         crate::input_to_egui(
             self.egui_ctx.pixels_per_point(),
-            &event,
+            event,
             self.clipboard.as_mut(),
             &mut self.input_state,
         );
@@ -523,10 +539,19 @@ impl EguiGlium {
             .unwrap_or_else(|| self.egui_ctx.pixels_per_point());
 
         self.input_state.raw.time = Some(self.start_time.elapsed().as_nanos() as f64 * 1e-9);
-        self.input_state.raw.screen_rect = Some(Rect::from_min_size(
-            Default::default(),
-            screen_size_in_pixels(display) / pixels_per_point,
-        ));
+
+        // On Windows, a minimized window will have 0 width and height.
+        // See: https://github.com/rust-windowing/winit/issues/208
+        // This solves an issue where egui window positions would be changed when minimizing on Windows.
+        let screen_size = screen_size_in_pixels(display);
+        self.input_state.raw.screen_rect = if screen_size.x > 0.0 && screen_size.y > 0.0 {
+            Some(Rect::from_min_size(
+                Default::default(),
+                screen_size / pixels_per_point,
+            ))
+        } else {
+            None
+        };
 
         self.egui_ctx.begin_frame(self.input_state.raw.take());
     }
